@@ -32,6 +32,10 @@ if re.search(r'_MSC_FULL_VER', clg.linkgrammar_get_configuration()) and \
    not re.search(r'USE_SQLITE', clg.linkgrammar_get_configuration()):
     NO_SQLITE_ERROR = 'Library is not configures with SQLite support'
 
+NOT_COMPILED_WITH_PCRE2 = ''
+if not re.search(r'HAVE_PCRE2_H', clg.linkgrammar_get_configuration()):
+   NOT_COMPILED_WITH_PCRE2 = 'Library not configured with PCRE2 support'
+
 # Show the location and version of the bindings modules
 for imported_module in 'linkgrammar$', 'clinkgrammar', '_clinkgrammar', 'lg_testutils':
     module_found = False
@@ -77,11 +81,11 @@ class AADictionaryTestCase(unittest.TestCase):
 
         save_stderr = divert_start(2)
         self.assertRaises(LG_DictionaryError, Dictionary, dummy_lang + '1')
-        self.assertIn(dummy_lang + '1', save_stderr.divert_end())
+        self.assertIn(dummy_lang + '1', str(save_stderr.divert_end()))
 
         save_stderr = divert_start(2)
         self.assertRaises(LG_Error, Dictionary, dummy_lang + '2')
-        self.assertIn(dummy_lang + '2', save_stderr.divert_end())
+        self.assertIn(dummy_lang + '2', str(save_stderr.divert_end()))
 
 # Check absolute and relative dictionary access.
 # Check also that the dictionary language is set correctly.
@@ -506,7 +510,7 @@ class EErrorFacilityTestCase(unittest.TestCase):
         dummy_lang = "a dummy dict name (bad param test)"
         self.assertRaises(LG_Error, Dictionary, dummy_lang)
         LG_Error.printall(self.error_handler_test, self) # grab a valid errinfo
-        #self.assertIn(dummy_lang, save_stderr.divert_end())
+        #self.assertIn(dummy_lang, str(save_stderr.divert_end()))
         self.assertRaisesRegexp(TypeError, "must be an integer",
                                 self.__class__.handler["default"],
                                 self.errinfo, "bad param")
@@ -520,7 +524,7 @@ class EErrorFacilityTestCase(unittest.TestCase):
             self.param_ok = False
             save_stdout = divert_start(1) # Note: Handler parameter is stdout
             self.__class__.handler["default"](self.errinfo, 1)
-            self.assertIn(dummy_lang, save_stdout.divert_end())
+            self.assertIn(dummy_lang, str(save_stdout.divert_end()))
             self.param_ok = True
         except (TypeError, ValueError):
             self.assertTrue(self.param_ok)
@@ -598,7 +602,7 @@ class EErrorFacilityTestCase(unittest.TestCase):
         dummy_lang = "a dummy dict name (default handler test)"
         save_stderr = divert_start(2)
         self.assertRaises(LG_Error, Dictionary, dummy_lang)
-        self.assertIn(dummy_lang, save_stderr.divert_end())
+        self.assertIn(dummy_lang, str(save_stderr.divert_end()))
         self.assertEqual(self.errinfo, "dummy")
 
 class FSATsolverTestCase(unittest.TestCase):
@@ -1134,6 +1138,39 @@ class YGenerationTestCase(unittest.TestCase):
         self.assertTrue(len(linkages) > 0, "No linkages")
 
 
+class ZANYAMYTestCase(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        if NOT_COMPILED_WITH_PCRE2 == '':
+           cls.amy_dict = Dictionary(lang='amy')
+           cls.amy_po = ParseOptions(display_morphology=True, linkage_limit=20000)
+        cls.any_dict = Dictionary(lang='any')
+        cls.any_po = ParseOptions(display_morphology=False, linkage_limit=200)
+
+    @classmethod
+    def tearDownClass(cls):
+        if NOT_COMPILED_WITH_PCRE2 == '':
+            del cls.amy_dict
+        del cls.any_dict
+
+    def find_num_linkages(self, sentense_text, dict, po):
+        return len(Sentence(sentense_text, dict, po).parse())
+
+    @unittest.skipIf(NOT_COMPILED_WITH_PCRE2, NOT_COMPILED_WITH_PCRE2)
+    def test_amy_num_linkages(self):
+       self.assertEqual(5292, self.find_num_linkages('this is a test', self.amy_dict, self.amy_po))
+
+    @unittest.skipIf(NOT_COMPILED_WITH_PCRE2, NOT_COMPILED_WITH_PCRE2)
+    def test_amy(self):
+        linkage_testfile(self, self.amy_dict, self.amy_po)
+
+    def test_any_num_linkages(self):
+       self.assertEqual(156, self.find_num_linkages('this is a test', self.any_dict, self.any_po))
+
+    def test_any(self):
+        linkage_testfile(self, self.any_dict, self.any_po)
+
+
 class ZENConstituentsCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -1252,9 +1289,16 @@ class ZRULangTestCase(unittest.TestCase):
              '.', 'RIGHT-WALL'])
 
 
+# The Thai 4.0.affix files currently contain strippable affixes that are
+# not in the dict. This causes an annoying multiline error output that are
+# filtered out here using divert().
 class ZTHLangTestCase(unittest.TestCase):
     def test_thai(self):
+        save_stderr = divert_start(2)
         linkage_testfile(self, Dictionary(lang='th'), ParseOptions())
+        for line in save_stderr.divert_end().decode().split("\n"):
+           if 'Token(s) not in the dictionary' not in line:
+              print(line)
 
 
 class ZXDictDialectTestCase(unittest.TestCase):
@@ -1441,6 +1485,7 @@ def linkage_testfile(self, lgdict, popt, desc=''):
         else:
             self.fail('\nTest file "{}": Invalid opcode "{}" (ord={})'.format(testfile, line[0], ord(line[0])))
 
+    self.assertIsNotNone(last_opcode, "Missing opcode in " + testfile)
     self.assertIn(last_opcode, 'OCP', "Missing result comparison in " + testfile)
 
 def warning(*msg):
@@ -1471,7 +1516,7 @@ class divert_start(object):
         os.close(self.savedfd)
         os.unlink(self.filename)
         self.filename = None
-        return str(content)
+        return content
 
     __del__ = divert_end
 
