@@ -40,10 +40,11 @@ class ParseOptions(object):
                  spell_guess=False,
                  use_sat=False,
                  max_parse_time=-1,
-                 disjunct_cost=2.7,
+                 disjunct_cost=None,
                  repeatable_rand=True,
                  test='',
                  debug='',
+                 dialect='',
                  ):
 
         self._obj = clg.parse_options_create()
@@ -58,10 +59,12 @@ class ParseOptions(object):
         self.spell_guess = spell_guess
         self.use_sat = use_sat
         self.max_parse_time = max_parse_time
-        self.disjunct_cost = disjunct_cost
+        if disjunct_cost is not None:
+            self.disjunct_cost = disjunct_cost
         self.repeatable_rand = repeatable_rand
         self.test = test
         self.debug = debug
+        self.dialect = dialect
 
     # Allow only the attribute names listed below.
     def __setattr__(self, name, value):
@@ -70,11 +73,6 @@ class ParseOptions(object):
             raise TypeError('Unknown parse option "{}".'.format(name))
         super(ParseOptions, self).__setattr__(name, value)
 
-    def __del__(self):
-        if hasattr(self, '_obj'):
-            clg.parse_options_delete(self._obj)
-            del self._obj
-
     @property
     def test(self):
         return clg.parse_options_get_test(self._obj)
@@ -82,7 +80,7 @@ class ParseOptions(object):
     @test.setter
     def test(self, value):
         if not isinstance(value, str):
-            raise TypeError("islands_ok must be set to a string")
+            raise TypeError("test must be set to a string")
         return clg.parse_options_set_test(self._obj, value)
 
     @property
@@ -92,9 +90,19 @@ class ParseOptions(object):
     @debug.setter
     def debug(self, value):
         if not isinstance(value, str):
-            raise TypeError("islands_ok must be set to a string")
+            raise TypeError("debug must be set to a string")
         return clg.parse_options_set_debug(self._obj, value)
 
+    @property
+    def dialect(self):
+        return clg.parse_options_get_dialect(self._obj)
+
+    @dialect.setter
+    def dialect(self, value):
+        if not isinstance(value, str):
+            raise TypeError("dialect must be set to a string")
+        return clg.parse_options_set_dialect(self._obj, value)
+
     @property
     def verbosity(self):
         """
@@ -106,9 +114,9 @@ class ParseOptions(object):
     @verbosity.setter
     def verbosity(self, value):
         if not isinstance(value, int):
-            raise TypeError("Verbosity must be set to an integer")
-        if value not in range(0,120):
-            raise ValueError("Verbosity levels can be any integer between 0 and 120 inclusive")
+            raise TypeError("verbosity must be set to a nonnegative integer")
+        if value < 0:
+            raise ValueError("verbosity must be set to a nonnegative integer")
         clg.parse_options_set_verbosity(self._obj, value)
 
     @property
@@ -125,9 +133,9 @@ class ParseOptions(object):
     @linkage_limit.setter
     def linkage_limit(self, value):
         if not isinstance(value, int):
-            raise TypeError("Linkage Limit must be set to an integer")
+            raise TypeError("linkage_limit must be set to an integer")
         if value < 0:
-            raise ValueError("Linkage Limit must be positive")
+            raise ValueError("linkage_limit must be positive")
         clg.parse_options_set_linkage_limit(self._obj, value)
 
     @property
@@ -142,7 +150,7 @@ class ParseOptions(object):
     @disjunct_cost.setter
     def disjunct_cost(self, value):
         if not isinstance(value, float):
-            raise TypeError("Distjunct cost must be set to a float")
+            raise TypeError("disjunct_cost must be set to a float")
         clg.parse_options_set_disjunct_cost(self._obj, value)
 
     @property
@@ -405,7 +413,7 @@ class Link(object):
 
     def __eq__(self, other):
         return self.left_word == other.left_word and self.left_label == other.left_label and \
-               self.right_word == other.right_word and self.right_label == other.right_label
+            self.right_word == other.right_word and self.right_label == other.right_label
 
     def __str__(self):
         if self.left_label == self.right_label:
@@ -445,7 +453,6 @@ class Linkage(object):
 
     __bool__ = __nonzero__      # Account python3
 
-
     def num_of_words(self):
         return clg.linkage_get_num_words(self._obj)
 
@@ -506,6 +513,7 @@ class Linkage(object):
 class LG_TimerExhausted(LG_Error):
     pass
 
+
 class Sentence(object):
     """
     sent = Sentence("This is a test.", Dictionary(), ParseOptions())
