@@ -25,6 +25,7 @@
 #include "post-process/post-process.h"  // post_process_new
 #include "prepare/exprune.h"
 #include "string-set.h"
+#include "tokenize//wordgraph.h"        // wordgraph_delete
 #include "resources.h"
 #include "sat-solver/sat-encoder.h"
 #include "tokenize/tokenize.h"
@@ -287,11 +288,11 @@ int parse_options_get_linkage_limit(Parse_Options opts)
 	return opts->linkage_limit;
 }
 
-void parse_options_set_disjunct_cost(Parse_Options opts, double dummy)
+void parse_options_set_disjunct_cost(Parse_Options opts, float dummy)
 {
 	opts->disjunct_cost = dummy;
 }
-double parse_options_get_disjunct_cost(Parse_Options opts)
+float parse_options_get_disjunct_cost(Parse_Options opts)
 {
 	return opts->disjunct_cost;
 }
@@ -558,7 +559,7 @@ void sentence_delete(Sentence sent)
 	pool_delete(sent->X_node_pool);
 
 	/* Usually the memory pools created in build_disjuncts_for_exp() are
-	 * deleted in build_sentence_disjuncts(). Delete them here inn case
+	 * deleted in build_sentence_disjuncts(). Delete them here in case
 	 * build_disjuncts_for_exp() is directly called. */
 	if (sent->Clause_pool != NULL)
 	{
@@ -566,9 +567,12 @@ void sentence_delete(Sentence sent)
 		pool_delete(sent->Tconnector_pool);
 	}
 
-	if (IS_DB_DICT(sent->dict))
+	// This is a hack. Should just ask the backend to "do the right
+	// thing".
+	if (IS_SQL_DICT(sent->dict))
 	{
 #if 0 /* Cannot reuse in case a previous sentence is not deleted yet. */
+		We could fix this by putting a use-count in the dict.
 		condesc_reuse(sent->dict);
 #endif
 		pool_reuse(sent->dict->Exp_pool);
@@ -623,7 +627,7 @@ int sentence_num_violations(Sentence sent, LinkageIdx i)
 	return sent->lnkages[i].lifo.N_violations;
 }
 
-double sentence_disjunct_cost(Sentence sent, LinkageIdx i)
+float sentence_disjunct_cost(Sentence sent, LinkageIdx i)
 {
 	if (!sent) return 0.0;
 
@@ -694,9 +698,9 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 
 	resources_reset(opts->resources);
 
-	/* When the SQL dict is used, expressions are read on demand, so
-	 * the connector descriptor table is not yet ready at this point. */
-	if (IS_DB_DICT(sent->dict))
+	/* When a dynamic dictionary is used, expressions are read on demand,
+	 * so the connector descriptor table is not yet ready at this point. */
+	if (IS_DYNAMIC_DICT(sent->dict))
 		condesc_setup(sent->dict);
 
 	for (WordIdx w = 0; w < sent->length; w++)
