@@ -18,10 +18,11 @@
 #include "dict-api.h"
 #include "dict-defines.h"
 #include "dict-impl.h"
-#include "regex-morph.h"
 #include "dict-structures.h"
-#include "string-set.h"
+#include "print/print-util.h"           // patch_subscript_mark
+#include "regex-morph.h"
 #include "string-id.h"
+#include "string-set.h"
 #include "utilities.h"
 
 /* ======================================================================= */
@@ -66,10 +67,9 @@ int callGetLocaleInfoEx(LPCWSTR lpLocaleName, LCTYPE LCType, LPWSTR lpLCData, in
 
 const char *linkgrammar_get_dict_define(Dictionary dict, const char *name)
 {
-	if (IS_DB_DICT(dict)) return NULL; /* Not supported yet */
-	unsigned int id = string_id_lookup(name, dict->define.set);
+	unsigned int id = string_id_lookup(name, dict->dfine.set);
 	if (id == 0) return NULL;
-	return dict->define.value[id - 1];
+	return dict->dfine.value[id - 1];
 }
 
 /* ======================================================================= */
@@ -304,8 +304,7 @@ const char * linkgrammar_get_dict_version(Dictionary dict)
 		return dict->version;
 	}
 
-	/* Original code is left for backward compatibility. Note that the
-	 * check for now version should be moved up if it is ever removed. */
+	/* Original code is left for backward compatibility. */
 
 	char * ver;
 	char * p;
@@ -335,14 +334,14 @@ const char * linkgrammar_get_dict_version(Dictionary dict)
 	return dict->version;
 }
 
-double linkgrammar_get_dict_max_disjunct_cost(Dictionary dict)
+float linkgrammar_get_dict_max_disjunct_cost(Dictionary dict)
 {
 	return dict->default_max_disjunct_cost;
 }
 
 /* ======================================================================= */
 
-static void dictionary_setup_locale(Dictionary dict)
+void dictionary_setup_locale(Dictionary dict)
 {
 	/* Get the locale for the dictionary. The first one of the
 	 * following which exists, is used:
@@ -405,9 +404,9 @@ static bool dictionary_setup_max_disjunct_cost(Dictionary dict)
 	else
 	{
 		float disjunct_cost_value;
-		if (!strtodC(disjunct_cost_str, &disjunct_cost_value))
+		if (!strtofC(disjunct_cost_str, &disjunct_cost_value))
 		{
-			prt_error("Error: %s: Invalid cost \"%s\"", LG_DISJUNCT_COST,
+			prt_error("Error: %s: Invalid cost \"%s\"\n", LG_DISJUNCT_COST,
 			          disjunct_cost_str);
 			return false;
 		}
@@ -444,13 +443,13 @@ bool dictionary_setup_defines(Dictionary dict)
 
 	dict->shuffle_linkages = false;
 
-	if (!dictionary_setup_max_disjunct_cost(dict)) return false;
 	dictionary_setup_locale(dict);
 
+	if (!dictionary_setup_max_disjunct_cost(dict)) return false;
+
 	return true;
 }
 
-/* ======================================================================= */
 /* ======================================================================= */
 
 /* The affix dictionary is represented as a dynamically allocated array with
@@ -458,21 +457,13 @@ bool dictionary_setup_defines(Dictionary dict)
  * has a pointer to an array of strings which are the punctuation/affix
  * names. */
 
-const char * afdict_classname[] = { AFDICT_CLASSNAMES };
-
 /** initialize the affix class table */
 void afclass_init(Dictionary dict)
 {
-	size_t i;
+	const size_t sz = sizeof(*dict->afdict_class) * AFDICT_NUM_ENTRIES;
 
-	dict->afdict_class =
-		malloc(sizeof(*dict->afdict_class) * AFDICT_NUM_ENTRIES);
-	for (i = 0; i < AFDICT_NUM_ENTRIES; i++)
-	{
-		dict->afdict_class[i].mem_elems = 0;
-		dict->afdict_class[i].length = 0;
-		dict->afdict_class[i].string = NULL;
-	}
+	dict->afdict_class = malloc(sz);
+	memset(dict->afdict_class, 0, sz);
 }
 
 /**
@@ -481,7 +472,7 @@ void afclass_init(Dictionary dict)
  */
 Afdict_class * afdict_find(Dictionary afdict, const char * con, bool notify_err)
 {
-	const char ** ac;
+	const char * const * ac;
 
 	for (ac = afdict_classname;
 	     ac < &afdict_classname[ARRAY_SIZE(afdict_classname)]; ac++)
@@ -508,7 +499,7 @@ void affix_list_add(Dictionary afdict, Afdict_class * ac,
 		size_t new_sz;
 		ac->mem_elems += AFFIX_COUNT_MEM_INCREMENT;
 		new_sz = ac->mem_elems * sizeof(const char *);
-		ac->string = (char const **) realloc((void *)ac->string, new_sz);
+		ac->string = realloc(ac->string, new_sz);
 	}
 	ac->string[ac->length] = string_set_add(affix, afdict->string_set);
 	ac->length++;
@@ -528,7 +519,8 @@ static int revcmplen(const void *a, const void *b)
  * The saved affixes don't include the infix mark.
  */
 static void get_dict_affixes(Dictionary dict, Dict_node * dn,
-                             char infix_mark, char * w_last)
+                             char infix_mark,
+                             const char** plast, size_t *lastlen)
 {
 	const char *w;         /* current dict word */
 	const char *w_sm;      /* SUBSCRIPT_MARK position in the dict word */
@@ -536,37 +528,32 @@ static void get_dict_affixes(Dictionary dict, Dict_node * dn,
 	Dictionary afdict = dict->affix_table;
 
 	if (dn == NULL) return;
-	get_dict_affixes(dict, dn->right, infix_mark, w_last);
+	get_dict_affixes(dict, dn->right, infix_mark, plast, lastlen);
 
 	w = dn->string;
-	w_sm = strrchr(w, SUBSCRIPT_MARK);
+	w_sm = get_word_subscript(w);
 	w_len = (NULL == w_sm) ? strlen(w) : (size_t)(w_sm - w);
-	if (w_len > MAX_WORD)
-	{
-		prt_error("Error: word '%s' too long (%zu), program may malfunction\n",
-		          w, w_len);
-		w_len = MAX_WORD;
-	}
-	/* (strlen(w_last) can be cached for speedup) */
-	if ((strlen(w_last) != w_len) || (0 != strncmp(w_last, w, w_len)))
-	{
-		strncpy(w_last, w, w_len);
-		w_last[w_len] = '\0';
 
-		if (infix_mark == w_last[0])
+	if (*lastlen != w_len || 0 != strncmp(*plast, w, w_len))
+	{
+		char* wtrunc = strdupa(w);
+		wtrunc[w_len] = '\0';
+
+		if (infix_mark == w[0])
 		{
-			affix_list_add(afdict, &afdict->afdict_class[AFDICT_SUF], w_last+1);
+			affix_list_add(afdict, &afdict->afdict_class[AFDICT_SUF], wtrunc+1);
 		}
-		else
-		if (infix_mark == w_last[w_len-1])
+		else if (infix_mark == w[w_len-1])
 		{
-			w_last[w_len-1] = '\0';
-			affix_list_add(afdict, &afdict->afdict_class[AFDICT_PRE], w_last);
-			w_last[w_len-1] = infix_mark;
+			wtrunc[w_len-1] = '\0';
+			affix_list_add(afdict, &afdict->afdict_class[AFDICT_PRE], wtrunc);
 		}
+
+		*plast = w;
+		*lastlen = w_len;
 	}
 
-	get_dict_affixes(dict, dn->left, infix_mark, w_last);
+	get_dict_affixes(dict, dn->left, infix_mark, plast, lastlen);
 }
 
 /**
@@ -607,16 +594,23 @@ static void concat_class(Dictionary afdict, int classno)
  * Sort order:
  * 1. Longest base words first.
  * 2. Equal base words one after the other.
+ * 3. Regexes last, in file order.
  */
 static int split_order(const void *a, const void *b)
 {
 	const char * const *sa = a;
 	const char * const *sb = b;
 
-	size_t len_a = strcspn(*sb, subscript_mark_str());
-	size_t len_b = strcspn(*sa, subscript_mark_str());
+	size_t len_a = strcspn(*sa, subscript_mark_str());
+	size_t len_b = strcspn(*sb, subscript_mark_str());
+	bool is_regex_a = (get_affix_regex_cg(*sa) >= 0);
+	bool is_regex_b = (get_affix_regex_cg(*sb) >= 0);
 
-	int len_order = (int)(len_a - len_b);
+	if (is_regex_a && is_regex_b) return 0;
+	if (is_regex_a) return 1;
+	if (is_regex_b) return -1;
+
+	int len_order = (int)(len_b - len_a);
 	if (0 == len_order) return strncmp(*sa, *sb, len_a);
 
 	return len_order;
@@ -634,6 +628,7 @@ bool afdict_init(Dictionary dict)
 {
 	Afdict_class * ac;
 	Dictionary afdict = dict->affix_table;
+	bool error_found = false;
 
 	/* FIXME: read_entry() builds word lists in reverse order (can we
 	 * just create the list top-down without breaking anything?). Unless
@@ -661,7 +656,7 @@ bool afdict_init(Dictionary dict)
 		prt_error("Error: afdict_init: Invalid value for class %s in file %s"
 		          " (should have been one ASCII punctuation - ignored)\n",
 		          afdict_classname[AFDICT_INFIXMARK], afdict->name);
-		free((void *)ac->string);
+		free(ac->string);
 		ac->length = 0;
 		ac->mem_elems = 0;
 		ac->string = NULL;
@@ -673,8 +668,9 @@ bool afdict_init(Dictionary dict)
 		if ((0 == AFCLASS(afdict, AFDICT_PRE)->length) &&
 		    (0 == AFCLASS(afdict, AFDICT_SUF)->length))
 		{
-			char last_entry[MAX_WORD+1] = "";
-			get_dict_affixes(dict, dict->root, ac->string[0][0], last_entry);
+			const char *last = 0x0;
+			size_t len = 0;
+			get_dict_affixes(dict, dict->root, ac->string[0][0], &last, &len);
 		}
 		else
 		{
@@ -693,9 +689,6 @@ bool afdict_init(Dictionary dict)
 	ac = AFCLASS(afdict, AFDICT_SANEMORPHISM);
 	if (0 != ac->length)
 	{
-		int rc;
-
-		Regex_node *sm_re = malloc(sizeof(*sm_re));
 		dyn_str *rebuf = dyn_str_new();
 
 		/* The regex used to be converted to: ^((original-regex)b)+$
@@ -712,38 +705,124 @@ bool afdict_init(Dictionary dict)
 #else
 		dyn_strcat(rebuf, ")+$");
 #endif
-		sm_re->pattern = strdup(rebuf->str);
+
+		afdict->regex_root = regex_new(afdict_classname[AFDICT_SANEMORPHISM],
+		                               rebuf->str);
 		dyn_str_delete(rebuf);
 
-		afdict->regex_root = sm_re;
-		sm_re->name = strdup(afdict_classname[AFDICT_SANEMORPHISM]);
-		sm_re->re = NULL;
-		sm_re->next = NULL;
-		sm_re->neg = false;
-		rc = compile_regexs(afdict->regex_root, afdict);
-		if (rc) {
+		if (!compile_regexs(afdict->regex_root, afdict))
+		{
 			prt_error("Error: afdict_init: Failed to compile "
-			          "regex '%s' in file %s, return code %d\n",
-			          afdict_classname[AFDICT_SANEMORPHISM], afdict->name, rc);
-			return false;
+			          "regex /%s/ in file \"%s\"\n",
+			          afdict_classname[AFDICT_SANEMORPHISM], afdict->name);
+			error_found = true;
+		}
+		else
+		{
+			lgdebug(+D_AI, "%s regex %s\n", afdict_classname[AFDICT_SANEMORPHISM],
+			        afdict->regex_root->pattern);
 		}
-		lgdebug(+D_AI, "%s regex %s\n",
-		        afdict_classname[AFDICT_SANEMORPHISM], sm_re->pattern);
 	}
 
-	/* Sort the affix-classes of tokens to be stripped. */
-	/* Longer unit names must get split off before shorter ones.
-	 * This prevents single-letter splits from screwing things
-	 * up. e.g. split 7gram before 7am before 7m.
-	 * Another example: The ellipsis "..." must appear before the dot ".".
-	 */
-	afdict_classnum af[] = {AFDICT_UNITS, AFDICT_LPUNC, AFDICT_RPUNC, AFDICT_MPUNC};
-	for (size_t i = 0; i < ARRAY_SIZE(af); i++)
+	for (size_t i = 0; i < ARRAY_SIZE(affix_strippable); i++)
 	{
-		ac = AFCLASS(afdict, af[i]);
-		if (0 < ac->length)
+		ac = AFCLASS(afdict, affix_strippable[i]);
+		int *capture_group = alloca(ac->length * sizeof(*capture_group));
+		ac->Nregexes = 0;
+
+		if (AFDICT_UNITS != affix_strippable[i])
 		{
-			qsort(ac->string, ac->length, sizeof(char *), split_order);
+			/* Locate the affix regexes, find their capture
+			 * groups and count them for memory allocation. */
+			for (size_t n = 0;  n < ac->length; n++)
+			{
+				capture_group[n] = get_affix_regex_cg(ac->string[n]);
+				if (capture_group[n] < 0) continue;
+
+				ac->Nregexes++;
+			}
+
+			/* Allocate the regexes at the end of the existing string pointer
+			 * memory block, and compile them. */
+			if (ac->Nregexes > 0)
+			{
+				ac->regex = malloc(ac->Nregexes * sizeof(Regex_node *));
+				/* Zero-out to prevent uninitialized access on error. */
+				memset(ac->regex, 0, ac->Nregexes * sizeof(Regex_node *));
+
+				size_t re_index = 0;
+				for (size_t n = 0;  n < ac->length; n++)
+				{
+					if (capture_group[n] < 0) continue;
+
+					/* Extract the regex pattern. */
+					const size_t regex_len =
+						strlen(ac->string[n]) - (sizeof("//.\\N") - 1/* \0 */);
+					const char *pattern = strndupa(ac->string[n] + 1, regex_len);
+
+					ac->regex[re_index] = regex_new(ac->string[n], pattern);
+					ac->regex[re_index]->capture_group = capture_group[n];
+					if (!compile_regexs(ac->regex[re_index], afdict))
+					{
+						prt_error("Error: afdict_init: Class %s in file %s: "
+						          "regex \"%s\" Failed to compile\n",
+						          afdict_classname[affix_strippable[i]],
+						          afdict->name, ac->regex[re_index]->name);
+						return false;
+					}
+					re_index++;
+				}
+			}
+		}
+
+		/* Sort the affix-classes of tokens to be stripped. */
+		/* Longer unit names must get split off before shorter ones.
+		 * This prevents single-letter splits from screwing things
+		 * up. e.g. split 7gram before 7am before 7m.
+		 * Another example: The ellipsis "..." must appear before the dot ".".
+		 * Regexes are just being moved to the end, at the same order.
+		 */
+		for (size_t s = 0; s < ARRAY_SIZE(affix_strippable); s++)
+		{
+			ac = AFCLASS(afdict, affix_strippable[i]);
+			if (0 < ac->length)
+			{
+				qsort(ac->string, ac->length, sizeof(char *), split_order);
+			}
+		}
+	}
+
+	if (!IS_DYNAMIC_DICT(dict))
+	{
+		/* Validate that the strippable tokens are in the dict. UNITS are
+		 * assumed to be from the dict only. */
+		for (size_t i = 0; i < ARRAY_SIZE(affix_strippable); i++)
+		{
+			if (AFDICT_UNITS != affix_strippable[i])
+			{
+				ac = AFCLASS(afdict, affix_strippable[i]);
+				bool not_in_dict = false;
+
+				for (int n = 0;  n < ac->length - ac->Nregexes; n++)
+				{
+					if (!dictionary_word_is_known(dict, ac->string[n]))
+					{
+						if (!not_in_dict)
+						{
+							not_in_dict = true;
+							prt_error("Warning: afdict_init: Class %s in file %s: "
+							          "Token(s) not in the dictionary:",
+							          afdict_classname[affix_strippable[i]],
+							          afdict->name);
+						}
+
+						char *s = strdupa(ac->string[n]);
+						patch_subscript_mark(s);
+						prt_error(" \"%s\"", s);
+					}
+				}
+				if (not_in_dict) prt_error("\n");
+			}
 		}
 	}
 
@@ -770,7 +849,7 @@ bool afdict_init(Dictionary dict)
 		     ac < &afdict->afdict_class[ARRAY_SIZE(afdict_classname)]; ac++)
 		{
 			if (0 == ac->length) continue;
-			lgdebug(+0, "Class %s, %zu items:",
+			lgdebug(+0, "Class %s, %d items:",
 			        afdict_classname[ac-afdict->afdict_class], ac->length);
 			for (l = 0; l < ac->length; l++)
 				lgdebug(0, " '%s'", ac->string[l]);
@@ -779,6 +858,6 @@ bool afdict_init(Dictionary dict)
 		lg_error_flush();
 	}
 
-	return true;
+	return !error_found;
 }
 #undef D_AI
