@@ -147,6 +147,8 @@ void free_Exp(Exp *e)
 /* Returns the number of connectors in the expression e */
 int size_of_expression(Exp * e)
 {
+	if (NULL == e) return 0;
+
 	int size = 0;
 
 	if (e->type == CONNECTOR_type) return 1;
@@ -162,8 +164,10 @@ Exp *copy_Exp(Exp *e, Pool_desc *Exp_pool, Parse_Options opts)
 	Exp *new_e = pool_alloc(Exp_pool);
 
 	*new_e = *e;
-	if ((e->type != CONNECTOR_type) && (Exptag_dialect == e->tag_type))
-		new_e->cost += opts->dialect.cost_table[new_e->tag_id];
+	if (opts) {
+		if ((e->type != CONNECTOR_type) && (Exptag_dialect == e->tag_type))
+			new_e->cost += opts->dialect.cost_table[new_e->tag_id];
+	}
 
 #if 0 /* Not used - left here for documentation. */
 	new_e->operand_next = copy_Exp(e->operand_next, Exp_pool);
