@@ -12,7 +12,8 @@
 /*************************************************************************/
 
 #include <ctype.h>
-#include <math.h>                       // fabs
+#include <inttypes.h>                   // format macros
+#include <math.h>                       // fabsf signbit
 
 #include "api-structures.h"             // Parse_Options_s  (seems hacky to me)
 #include "dict-common.h"
@@ -27,22 +28,33 @@
 #include "regex-morph.h"
 #include "tokenize/tokenize.h"          // word_add
 #include "tokenize/word-structures.h"   // Word_struct
-#include "utilities.h"                  // GNU_UNUSED
+#include "utilities.h"                  // GNUC_UNUSED
 /* ======================================================================== */
 
 bool cost_eq(float cost1, float cost2)
 {
-	return (fabs(cost1 - cost2) < cost_epsilon);
+	return (fabsf(cost1 - cost2) < cost_epsilon);
 }
 
+/* Create a float constant with mantissa a and exponent s. */
+#define COMBINE(a, b, c) a ## b ## c
+#define EXP10F(a, s) COMBINE(a ## e, s, F)
 /**
- * Convert cost to a string with at most cost_max_dec_places decimal places.
+ * Convert \p cost to a string with COST_MAX_DEC_PLACES decimal places.
+ * Always use dot as a radix character.
+ * @return A static thread-local pointer to the result string.
  */
 const char *cost_stringify(float cost)
 {
 	static TLS char buf[16];
 
-	int l = snprintf(buf, sizeof(buf), "%.*f", cost_max_dec_places, cost);
+	const int scale = (int)EXP10F(1, COST_MAX_DEC_PLACES);
+	const bool sign = signbit(cost);
+	const float roundinc = (1 / EXP10F(2, COST_MAX_DEC_PLACES));
+	int c = (int)((fabsf(cost) + roundinc) * scale);
+
+	int l = snprintf(buf, sizeof(buf), "%s%d.%0*d", sign ? "-" : "",
+	                 c / scale, COST_MAX_DEC_PLACES, c - c / scale * scale);
 	if ((l < 0) || (l >= (int)sizeof(buf))) return "ERR_COST";
 
 	return buf;
@@ -146,7 +158,7 @@ static void get_expression_cost(const Exp *e, unsigned int *icost, float *dcost)
 			if (*icost > 4)
 			{
 				/* don't print too many [] levels */
-				*dcost = *icost;
+				*dcost = (float)*icost;
 				*icost = 1;
 			}
 			else
@@ -882,12 +894,7 @@ static size_t copy_quoted(const char *q, char *dst, const char *src, size_t len)
 
 static Regex_node *new_disjunct_regex_node(Regex_node *current, char *regpat)
 {
-	Regex_node *rn = malloc(sizeof(Regex_node));
-
-	rn->name = "Disjunct regex";
-	rn->pattern = strdup(regpat);
-	rn->re = NULL;
-	rn->neg = false;
+	Regex_node *rn = regex_new("Disjunct regex", regpat);
 	rn->next = current;
 
 	return rn;
@@ -1205,7 +1212,7 @@ static char *display_counts(const char *word, Dict_node *dn)
 	dyn_strcat(s, "matches:\n");
 	for (; dn != NULL; dn = dn->right)
 	{
-		append_string(s, "    %-*s %8zu  disjuncts",
+		append_string(s, "    %-*s %8"PRIu64" disjuncts",
 		              display_width(DJ_COL_WIDTH, dn->string), dn->string,
 		              count_disjunct_for_dict_node(dn));
 
