@@ -10,6 +10,8 @@
 /*                                                                       */
 /*************************************************************************/
 
+#include <threads.h>
+
 /**
  * Support for the regular-expression based token matching system
  * using standard POSIX regex, PCRE2 or C++.
@@ -47,7 +49,7 @@ LINK_BEGIN_DECLS
 #include "dict-common/regex-morph.h"
 LINK_END_DECLS
 
-/* ovector     - returned start and end offsets.
+/*
  * re_code     - compiled regex.
  * re_md       - match data.
  */
@@ -55,46 +57,34 @@ LINK_END_DECLS
 #if HAVE_PCRE2_H
 typedef struct {
 	pcre2_code *re_code;
-	pcre2_match_data* re_md;
 } reg_info;
 #endif
 
 #if HAVE_REGEX_H
 typedef struct {
 	regex_t re_code;
-	unsigned int Nre_md;
-	regmatch_t re_md[];
 } reg_info;
 #endif
 
 #if USE_CXXREGEX
 typedef struct {
 	std::regex *re_code;
-	std::cmatch re_md;
 } reg_info;
 #endif
 
 #define ERRBUFFLEN 120
 
+#define MAX_CAPTURE_GROUPS 10
+
 /* Define reg_comp(), reg_match() and reg_free(), and reg_span()
  * for the selected regex implementation. */
 
 /* ========================================================================= */
 
 #if HAVE_REGEX_H
-/**
- * Find an upper limit to the number of capture groups in the pattern.
- */
-static unsigned int max_capture_groups(const Regex_node *rn)
-{
-	if (rn->capture_group < 0) return 0;
 
-	unsigned int Nov = 1;
-	for (const char *p = rn->pattern; *p != '\0'; p++)
-		if (*p == '(') Nov++;
-
-	return Nov;
-}
+#define ALLOCTE_MATCH_DATA(var) \
+	regmatch_t mem_##var[MAX_CAPTURE_GROUPS]; regmatch_t *var = mem_##var
 
 /**
  * Compile the given regex..
@@ -110,13 +100,9 @@ static bool reg_comp(Regex_node *rn)
 #endif
 
 	const bool nosub = (rn->capture_group < 0);
-	const unsigned int Novector = nosub ? 0 : (max_capture_groups(rn) + 1);
 	int options = REG_EXTENDED|REG_ENHANCED|REG_GNU|(nosub ? REG_NOSUB : 0);
-
-	reg_info *re = malloc(sizeof(reg_info) + sizeof(regmatch_t) * 2 * Novector);
-	memset(re, 0, sizeof(reg_info)); /* No need to initialize re_md. */
-
-	re->Nre_md = Novector;
+	reg_info *re = malloc(sizeof(reg_info));
+	memset(re, 0, sizeof(reg_info));
 
 	int rc = regcomp(&re->re_code, rn->pattern, options);
 
@@ -134,11 +120,10 @@ static bool reg_comp(Regex_node *rn)
 	return true;
 }
 
-static bool reg_match(const char *s, const Regex_node *rn)
+static bool reg_match(const char *s, const Regex_node *rn, regmatch_t *re_md)
 {
-	reg_info *re = (reg_info *)rn->re;
-	size_t nmatch = (rn->capture_group < 0) ? 0 : re->Nre_md;
-	int rc = regexec(&re->re_code, s, nmatch, re->re_md, /*eflags*/0);
+	reg_info *re = rn->re;
+	int rc = regexec(&re->re_code, s, MAX_CAPTURE_GROUPS, re_md, /*eflags*/0);
 
 	if (rc == REG_NOMATCH) return false;
 	if (rc == 0) return true;
@@ -151,19 +136,18 @@ static bool reg_match(const char *s, const Regex_node *rn)
 	return false;
 }
 
-static void reg_span(Regex_node *rn)
+static void reg_span(Regex_node *rn, int *start, int *end, regmatch_t *re_md)
 {
 	int cgn = rn->capture_group;
-	reg_info *re = rn->re;
 
-	if (unlikely(cgn > rn->capture_group))
+	if (unlikely(cgn > MAX_CAPTURE_GROUPS))
 	{
-		rn->ovector[0] = rn->ovector[1] = -1;
+		*start = *end = -1;
 	}
 	else
 	{
-		rn->ovector[0] = re->re_md[cgn].rm_so;
-		rn->ovector[1] = re->re_md[cgn].rm_eo;
+		*start = re_md[cgn].rm_so;
+		*end = re_md[cgn].rm_eo;
 	}
 }
 
@@ -179,6 +163,25 @@ static void reg_free(Regex_node *rn)
 
 /* ========================================================================= */
 #if HAVE_PCRE2_H
+
+#define ALLOCTE_MATCH_DATA(var) \
+	pcre2_match_data *var = \
+	memcpy(alloca(re_md_template_size), re_md_template, re_md_template_size)
+
+static unsigned int re_md_template_size;
+static pcre2_match_data* re_md_template;
+
+#if HAVE_THREADS_H
+static once_flag call_once_flag = ONCE_FLAG_INIT;
+#endif // HAVE_THREADS_H
+static void alloc_match_data(void)
+{
+	re_md_template = pcre2_match_data_create(MAX_CAPTURE_GROUPS, NULL);
+	if (re_md_template == NULL)
+		return;
+	re_md_template_size = pcre2_get_match_data_size(re_md_template);
+}
+
 static bool reg_comp(Regex_node *rn)
 {
 	reg_info *re = rn->re = malloc(sizeof(reg_info));
@@ -192,10 +195,14 @@ static bool reg_comp(Regex_node *rn)
 	                            options, &rc, &erroffset, NULL);
 	if (re->re_code != NULL)
 	{
-		re->re_md = pcre2_match_data_create_from_pattern(re->re_code, NULL);
-		if (re->re_md == NULL)
+#if HAVE_THREADS_H
+		call_once(&call_once_flag, alloc_match_data);
+#else
+		alloc_match_data();
+#endif
+		if (0 == re_md_template_size)
 		{
-			prt_error("Error: pcre2_match_data_create failed\n");
+			prt_error("Error: pcre2_match_data_create() failed\n");
 			free(re);
 			return false;
 		}
@@ -212,13 +219,13 @@ static bool reg_comp(Regex_node *rn)
 	return false;
 }
 
-static int reg_match(const char *s, const Regex_node *rn)
+static int reg_match(const char *s, const Regex_node *rn, pcre2_match_data *re_md)
 {
 	reg_info *re = rn->re;
 
 	int rc = pcre2_match(re->re_code, (PCRE2_SPTR)s,
 	                     PCRE2_ZERO_TERMINATED, /*startoffset*/0,
-	                     PCRE2_NO_UTF_CHECK, re->re_md, NULL);
+	                     PCRE2_NO_UTF_CHECK, re_md, NULL);
 	if (rc == PCRE2_ERROR_NOMATCH) return false;
 	if (rc >= 0) return true;
 
@@ -230,28 +237,25 @@ static int reg_match(const char *s, const Regex_node *rn)
 	return false;
 }
 
-static void reg_span(Regex_node *rn)
+static void reg_span(Regex_node *rn, int *start, int *end, pcre2_match_data *re_md)
 {
 	int cgn = rn->capture_group;
-	reg_info *re = rn->re;
-	PCRE2_SIZE *ovector = pcre2_get_ovector_pointer(re->re_md);
 
-	if (unlikely(cgn >= (int)pcre2_get_ovector_count(re->re_md)))
+	if (unlikely(cgn >= MAX_CAPTURE_GROUPS))
 	{
-		rn->ovector[0] = rn->ovector[1] = -1;
+		*start = *end = -1;
 	}
 	else
 	{
-		rn->ovector[0] = (int)ovector[2*cgn];
-		rn->ovector[1] = (int)ovector[2*cgn + 1];
+		PCRE2_SIZE *ovector = pcre2_get_ovector_pointer(re_md);
+		*start = (int)ovector[2*cgn];
+		*end = (int)ovector[2*cgn + 1];
 	}
 }
 
 static void reg_free(Regex_node *rn)
 {
 	reg_info *re = rn->re;
-
-	pcre2_match_data_free(re->re_md);
 	pcre2_code_free(re->re_code);
 	free(re);
 	rn->re = NULL;
@@ -260,6 +264,10 @@ static void reg_free(Regex_node *rn)
 
 /* ========================================================================= */
 #if USE_CXXREGEX
+
+#define ALLOCTE_MATCH_DATA(var) \
+	std::cmatch mem_##var; std::cmatch *var = &mem_##var
+
 static bool reg_comp(Regex_node *rn)
 {
 	rn->re = new reg_info;
@@ -279,15 +287,15 @@ static bool reg_comp(Regex_node *rn)
 	return true;
 }
 
-static bool reg_match(const char *s, const Regex_node *rn)
+static bool reg_match(const char *s, const Regex_node *rn, std::cmatch *re_md)
 {
-	/* "nosub" not used, as no time difference found w/o using re->re_md. */
-	bool match;
+	/* "nosub" not used, as no time difference found w/o using re_md. */
+	bool match = false;
 	reg_info *re = (reg_info *)rn->re;
 
 	try
 	{
-		match = std::regex_search(s, re->re_md, *re->re_code);
+		match = std::regex_search(s, *re_md, *re->re_code);
 	}
 	catch (const std::regex_error& e)
 	{
@@ -299,19 +307,18 @@ static bool reg_match(const char *s, const Regex_node *rn)
 	return match;
 }
 
-static void reg_span(Regex_node *rn)
+static void reg_span(Regex_node *rn, int *start, int *end, std::cmatch *re_md)
 {
 	int cgn = rn->capture_group;
-	std::cmatch re_md = ((reg_info *)rn->re)->re_md;
 
-	if (unlikely(cgn >= (int)re_md.size()))
+	if (unlikely(cgn >= (int)re_md->size()))
 	{
-		rn->ovector[0] = rn->ovector[1] = -1;
+		*start = *end = -1;
 	}
 	else
 	{
-		rn->ovector[0] = (int)re_md.position(cgn);
-		rn->ovector[1] = rn->ovector[0] + (int)re_md.length(cgn);
+		*start = (int)re_md->position(cgn);
+		*end = *start + (int)re_md->length(cgn);
 	}
 }
 
@@ -337,7 +344,8 @@ static void reg_free(Regex_node *rn)
 static bool check_capture_group(const Regex_node *rn)
 {
 	if (rn->capture_group <= 0) return true;
-	assert(rn->capture_group <= 9, "Bogus capture group %d", rn->capture_group);
+	assert(rn->capture_group < MAX_CAPTURE_GROUPS,
+		"Bogus capture group %d", rn->capture_group);
 
 	Regex_node check_cg = *rn;
 	const size_t pattern_len = strlen(rn->pattern);
@@ -398,11 +406,13 @@ bool compile_regexs(Regex_node *rn, Dictionary dict)
 #define D_MRE 6
 const char *match_regex(const Regex_node *rn, const char *s)
 {
+	ALLOCTE_MATCH_DATA(re_md);
+
 	while (rn != NULL)
 	{
 		if (rn->re == NULL) continue; // Make sure the regex has been compiled.
 
-		if (reg_match(s, rn))
+		if (reg_match(s, rn, re_md))
 		{
 			lgdebug(+D_MRE, "%s%s %s\n", &"!"[!rn->neg], rn->name, s);
 			if (!rn->neg)
@@ -429,20 +439,19 @@ const char *matchspan_regex(Regex_node *rn, const char *s,
                             int *start, int *end)
 {
 	assert(rn->capture_group >= 0, "No capture");
+	ALLOCTE_MATCH_DATA(re_md);
 
 	while (rn != NULL)
 	{
 		if (rn->re == NULL) continue; // Make sure the regex has been compiled.
 
-		if (reg_match(s, rn))
+		if (reg_match(s, rn, re_md))
 		{
 			lgdebug(+D_MRE, "%s%s %s\n", &"!"[!rn->neg], rn->name, s);
 			if (!rn->neg)
 			{
-				reg_span(rn);
-				lgdebug(+D_MRE, " [%d, %d)\n", rn->ovector[0], rn->ovector[1]);
-				*start = rn->ovector[0];
-				*end = rn->ovector[1];
+				reg_span(rn, start, end, re_md);
+				lgdebug(+D_MRE, " [%d, %d)\n", *start, *end);
 
 				if (unlikely(*start == -1))
 				{
