@@ -136,7 +136,7 @@ static Gword *wordgraph_link_placeholder(Sentence sent, Gword *w)
 {
 	Gword *new_word;
 
-	new_word = gword_new(sent, "");
+	new_word = gword_new(sent, "PLACE_HOLDER");
 	new_word->status |= WS_PL;
 	new_word->label = "PH";
 	new_word->start = w->start;
@@ -894,7 +894,7 @@ const char * linkage_get_disjunct_str(const Linkage linkage, WordIdx w)
 	return linkage->disjunct_list_str[w];
 }
 
-double linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
+float linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
 {
 	Disjunct *dj;
 
@@ -903,8 +903,13 @@ double linkage_get_disjunct_cost(const Linkage linkage, WordIdx w)
 	dj = linkage->chosen_disjuncts[w];
 
 	/* dj may be null, if the word didn't participate in the parse. */
-	if (dj) return dj->cost;
-	return 0.0;
+	if (NULL == dj)
+		return 0.0;
+
+	if (dj->is_category)
+		return dj->category[0].cost;
+
+	return dj->cost;
 }
 
 const char * linkage_get_word(const Linkage linkage, WordIdx w)
@@ -921,7 +926,7 @@ int linkage_unused_word_cost(const Linkage linkage)
 	return linkage->lifo.unused_word_cost;
 }
 
-double linkage_disjunct_cost(const Linkage linkage)
+float linkage_disjunct_cost(const Linkage linkage)
 {
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage) return 0.0;
