@@ -128,7 +128,7 @@ struct count_context_s
 /* Each thread will get it's own version of the `kept_table`.
  * If the program creates zillions of threads, then there will
  * be a mem-leak if this table is not released when each thread
- * exists. This code arranges so that `free_tls_table` is called
+ * exits. This code arranges so that `free_tls_table` is called
  * when the thread exists.
  */
 static void free_tls_table(void* ptr_to_table)
@@ -369,10 +369,12 @@ static void table_stat(count_context_t *ctxt)
 			assert(t->hash != 0, "Invalid hash value: 0");
 			assert((hist_total(&t->count)>=0)&&(hist_total(&t->count) <= INT_MAX),
 			       "Invalid count %"COUNT_FMT, hist_total(&t->count));
-			assert(t->l_id < (int)ctxt->sent->length ||
+			assert((ctxt->table_lrcnt[0].sz == 0) ||
+			       t->l_id < (int)ctxt->sent->length ||
 			       ((t->l_id >= 255)&&(t->l_id < (int)ctxt->table_lrcnt[0].sz)),
 			       "invalid l_id %d", t->l_id);
-			assert(t->r_id <= (int)ctxt->sent->length ||
+			assert((ctxt->table_lrcnt[1].sz == 0) ||
+			       t->r_id <= (int)ctxt->sent->length ||
 			       ((t->r_id > 255)&&(t->r_id < (int)ctxt->table_lrcnt[1].sz)),
 			       "invalid r_id %d", t->r_id);
 		}
@@ -1254,8 +1256,8 @@ static Count_bin do_count(
 		size_t mlb = form_match_list(mchxt, w, le, lw, fml_re, rw, mlcl, mlcr);
 
 #ifdef VERIFY_MATCH_LIST
-		unsigned int id = get_match_list_element(mchxt, mlb) ?
-		                  get_match_list_element(mchxt, mlb)->match_id : 0;
+		Disjunct *od = get_match_list_element(mchxt, mlb);
+		uint16_t id = od ? od->match_id : 0;
 #endif
 
 		for (size_t mle = mlb; get_match_list_element(mchxt, mle) != NULL; mle++)
@@ -1263,6 +1265,9 @@ static Count_bin do_count(
 			COUNT_COST(ctxt->count_cost[1]++;)
 
 			Disjunct *d = get_match_list_element(mchxt, mle);
+#ifdef VERIFY_MATCH_LIST
+			assert(id == d->match_id, "Modified id (%u!=%u)", id, d->match_id);
+#endif
 			bool Lmatch = d->match_left;
 			bool Rmatch = d->match_right;
 			w_Count_bin leftcount = NO_COUNT;
@@ -1296,10 +1301,6 @@ static Count_bin do_count(
 				}
 			}
 
-#ifdef VERIFY_MATCH_LIST
-			assert(id == d->match_id, "Modified id (%u!=%u)", id, d->match_id);
-#endif
-
 			for (unsigned int lnull_cnt = lnull_start; lnull_cnt <= lnull_end; lnull_cnt++)
 			{
 				COUNT_COST(ctxt->count_cost[2]++;)
