@@ -200,6 +200,9 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 {
 	size_t i;
 
+	if (IS_GENERATION(sent->dict))
+		create_wildcard_word_disjunct_list(sent, opts);
+
 	build_sentence_disjuncts(sent, opts->disjunct_cost, opts);
 	if (verbosity_level(D_PREP))
 	{
@@ -208,7 +211,6 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 	}
 	print_time(opts, "Built disjuncts");
 
-	bool wildcard_word_found = false;
 	for (i=0; i<sent->length; i++)
 	{
 		sent->word[i].d = eliminate_duplicate_disjuncts(sent->word[i].d, false);
@@ -219,15 +221,16 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 				/* Also with different word_string. */
 				sent->word[i].d = eliminate_duplicate_disjuncts(sent->word[i].d, true);
 
-				int n = 0;
+				/* XXX This ordinal numbering is just plain wrong.
+				 * Most of the disjuncts have already been pruned away,
+				 * and what is left here can no longer match the ordinal
+				 * numbering in the wild-card.  XXX FIXME, although the fix
+				 * is not obvious. FWIW, this is not used anywhere, except
+				 * to report unused disjuncts if the -u flag is specified.
+				 */
+				int nord = 0;
 				for (Disjunct *d = sent->word[i].d; d != NULL; d = d->next)
-					d->ordinal = n++;
-
-				if (!wildcard_word_found)
-				{
-					wildcard_word_found = true;
-					create_wildcard_word_disjunct_list(sent, opts);
-				}
+					d->ordinal = nord++;
 			}
 			else
 			{
