@@ -271,7 +271,8 @@ char * linkage_print_disjuncts(const Linkage linkage)
 		if (NULL == dj) dj = "";
 		cost = (float)linkage_get_disjunct_cost(linkage, w);
 
-		append_string(s, "%*s    % 4.3f  %s\n", pad, infword, cost, dj);
+		append_string(s, "%*s    %6s  %s\n",
+		              pad, infword, cost_stringify(cost), dj);
 	}
 	return dyn_str_take(s);
 }
