@@ -41,7 +41,7 @@ static struct
 #endif
 	int echo_on;
 	Cost_Model_type cost_model;
-	double max_cost;
+	float max_cost;
 	int screen_width;
 	int display_on;
 	ConstituentDisplayStyle display_constituents;
@@ -62,12 +62,13 @@ static const char *value_type[] =
 	"(integer) ", "(Boolean) ", "(float) ", "(string) ", "(command) ", ""
 };
 
-static int panic_variables_cmd(const Switch*, int);
-static int variables_cmd(const Switch*, int);
+static int clear_cmd(const Switch*, int);
+static int exit_cmd(const Switch*, int);
 static int file_cmd(const Switch*, int);
 static int help_cmd(const Switch*, int);
-static int exit_cmd(const Switch*, int);
 static int info_cmd(const Switch*, int);
+static int panic_variables_cmd(const Switch*, int);
+static int variables_cmd(const Switch*, int);
 
 Switch default_switches[] =
 {
@@ -110,6 +111,7 @@ Switch default_switches[] =
 	{"walls",      Bool, "Display wall words",              &local.display_walls},
 	{"width",      Int,  "The width of the display",        &local.screen_width},
 	{"wordgraph",  Int,  "Display sentence word-graph",     &local.display_wordgraph},
+	{"clear",      Cmd,  "Clear the AtomSpace cache",              clear_cmd},
 	{"exit",       Cmd,  "Exit the program",                       exit_cmd},
 	{"file",       Cmd,  "Read input from the specified filename", file_cmd},
 	{"help",       Cmd,  "List the commands and what they do",     help_cmd},
@@ -237,7 +239,7 @@ static const char *switch_value_string(const Switch *as)
 	switch (as->param_type)
 	{
 		case Float: /* Float point print! */
-			snprintf(buf, sizeof(buf), "%.2f", *((double *)as->ptr));
+			snprintf(buf, sizeof(buf), "%.3f", *((float *)as->ptr));
 			break;
 		case Bool:
 			/* FALLTHRU */
@@ -706,6 +708,11 @@ static int exit_cmd(const Switch *uc, int n)
 	return 'e';
 }
 
+static int clear_cmd(const Switch *uc, int n)
+{
+	return 'k';
+}
+
 static int file_cmd(const Switch *uc, int n)
 {
 	return 'f';
@@ -939,7 +946,7 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 		if (as[j].param_type == Float)
 		{
 			char *err;
-			double val = strtod(y, &err);
+			float val = strtof(y, &err);
 			if (('\0' == *y) || ('\0' != *err))
 			{
 				prt_error("Error: Invalid value \"%s\" for variable \"%s\". %s\n",
@@ -947,8 +954,8 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 				return -1;
 			}
 
-			*((double *) as[j].ptr) = val;
-			printf("%s set to %5.2f\n", as[j].string, val);
+			*((float *) as[j].ptr) = val;
+			printf("%s set to %5.3f\n", as[j].string, val);
 			return 'c';
 		}
 		else
