@@ -45,8 +45,10 @@ typedef struct
 {
 	const char* language;
 	unsigned int sentence_length;
-	unsigned int corpus_size; /* Limited to INT_MAX. */
+	unsigned int nsentences; /* Limited to INT_MAX. */
+	unsigned int nlinkages;  /* Limited to INT_MAX. */
 	bool display_disjuncts;
+	bool display_cost;
 	bool display_unused_disjuncts;
 	bool leave_subscripts;
 	bool unrepeatable_random;
@@ -75,11 +77,13 @@ static struct argp_option
 {
 	{"language", 'l', "language", 0, "Directory containing language definition."},
 	{"length", 's', "length", 0, "Sentence length. If 0 - get sentence template from stdin."},
-	{"count", 'c', "count", 0, "Count of number of sentences to generate."},
+	{"count", 'c', "count", 0, "Number of linkages to generate."},
+	{"print", 'k', "count", 0, "Number of sentences to print."},
 	{"explode", 'x', 0, 0, "Generate all wording samples per linkage."},
 	{"disjuncts", 'd', 0, 0, "Display linkage disjuncts."},
 	{"unused", 'u', 0, 0, "Display unused disjuncts."},
 	{"leave-subscripts", '.', 0, 0, "Don't remove word subscripts."},
+	{"show-cost", 'p', 0, 0, "Display linkage cost."},
 	{"random", 'r', 0, 0, "Use unrepeatable random numbers."},
 	{"no-walls", 'w'+128, 0, 0, "Don't use walls in wildcard words."},
 	{0, 0, 0, 0, "Library options:", 1},
@@ -411,11 +415,15 @@ static void getopt_parse(int ac, char *av[], struct argp_option *a_opt,
 			case 's': gp->sentence_length = (int)
 			             strtoi_errexit(av[optind-2], optarg, 0, MAX_SENTENCE-2);
 			          break;
-			case 'c': gp->corpus_size =
+			case 'c': gp->nlinkages =
+			             strtoi_errexit(av[optind-2], optarg, 1, INT_MAX);
+			          break;
+			case 'k': gp->nsentences =
 			             strtoi_errexit(av[optind-2], optarg, 1, INT_MAX);
 			          break;
 			case 'x': gp->explode = true; break;
 			case 'd': gp->display_disjuncts = true; break;
+			case 'p': gp->display_cost = true; break;
 			case 'u': gp->display_unused_disjuncts = true; break;
 			case '.': gp->leave_subscripts = true; break;
 			case 'r': gp->unrepeatable_random = true; break;
@@ -489,8 +497,10 @@ int main (int argc, char* argv[])
 	gen_parameters parms;
 	parms.language = "lt";
 	parms.sentence_length = 6;
-	parms.corpus_size = 50;
+	parms.nlinkages = 500;
+	parms.nsentences = 20;
 	parms.explode = false;
+	parms.display_cost = false;
 	parms.display_disjuncts = false;
 	parms.display_unused_disjuncts = false;
 	parms.leave_subscripts = false;
@@ -511,7 +521,8 @@ int main (int argc, char* argv[])
 	printf("#\n# Corpus for language: \"%s\"\n", parms.language);
 	if (parms.sentence_length != 0)
 		printf("# Sentence length: %u\n", parms.sentence_length);
-	printf("# Requested corpus size: %u\n", parms.corpus_size);
+	printf("# Requested number of linkages: %u\n", parms.nlinkages);
+	printf("# Requested number to print: %u\n", parms.nsentences);
 
 	// Force the system into generation mode by setting the "test"
 	// parse-option to "generate".
@@ -546,7 +557,7 @@ int main (int argc, char* argv[])
 		exit(0);
 	}
 
-	parse_options_set_linkage_limit(opts, parms.corpus_size);
+	parse_options_set_linkage_limit(opts, parms.nlinkages);
 
 	if (parms.sentence_length > 0)
 	{
@@ -619,7 +630,7 @@ int main (int argc, char* argv[])
 	// How many sentences to print per linkage.
 	// Print more than one only if explode flag set.
 	double samples = parms.explode ?
-		((double) parms.corpus_size) / ((double) num_linkages)
+		((double) parms.nsentences) / ((double) num_linkages)
 		: 1.0;
 
 	unsigned int num_printed = 0;
@@ -635,6 +646,9 @@ int main (int argc, char* argv[])
 		num_printed += print_sentences(catlist, linkage, nwords, words,
 		                               parms.leave_subscripts, samples);
 
+		if (parms.display_cost)
+			printf("# linkage-cost= %.3f\n", linkage_disjunct_cost(linkage));
+
 		if (parms.display_disjuncts)
 		{
 			char *disjuncts = linkage_print_disjuncts(linkage);
@@ -644,7 +658,7 @@ int main (int argc, char* argv[])
 
 		linkage_delete(linkage);
 
-		if (num_printed >= parms.corpus_size) break;
+		if (num_printed >= parms.nsentences) break;
 	}
 
 	free(unused_disjuncts);
