@@ -345,6 +345,7 @@ static const char *process_some_linkages(FILE *in, Sentence sent,
 				if ((verbosity > 0) && (!copts->batch_mode) && isatty_io)
 				{
 					fprintf(stdout, "Press RETURN for the next linkage.\n");
+					fflush(stdout);
 				}
 				char *rc = fget_input_string(use_prompt(verbosity),  stdin, stdout,
 				                             isatty_io, /*check_return*/true);
@@ -366,7 +367,7 @@ static int there_was_an_error(Label label, Sentence sent, Parse_Options opts)
 			batch_errors++;
 			return UNGRAMMATICAL;
 		}
-		if ((sentence_disjunct_cost(sent, 0) == 0.0) &&
+		if ((sentence_disjunct_cost(sent, 0) == 0.0F) &&
 			(label == PARSE_WITH_DISJUNCT_COST_GT_0)) {
 			batch_errors++;
 			return PARSE_WITH_DISJUNCT_COST_GT_0;
@@ -617,7 +618,7 @@ int main(int argc, char * argv[])
 	{
 		const char *locale =  setlocale(LC_NUMERIC, "C");
 		char *err;
-		double panic_max_cost = strtod(panic_max_cost_str, &err);
+		float panic_max_cost = strtof(panic_max_cost_str, &err);
 		setlocale(LC_NUMERIC, locale);
 
 		if ('\0' == *err)
@@ -742,6 +743,17 @@ int main(int argc, char * argv[])
 		if ('c' == command) continue; /* It was another command */
 		if (-1 == command) continue;  /* It was a bad command */
 
+		/* Not sure how else to handle this!? */
+		/* We need the dict, and the x_issue_special_command code
+		 * has it, but ... doesn't pass it to the command.
+		 * So we handle it here, based on the return code. Yikes!
+		 */
+		if ('k' == command)
+		{
+			dictionary_clear_cache(dict);
+			continue;
+		}
+
 		/* We have to handle the !file command inline; it's too hairy
 		 * otherwise ... */
 		if ('f' == command)
@@ -913,6 +925,12 @@ open_error:
 						parse_options_set_min_null_count(opts, 1);
 						parse_options_set_max_null_count(opts, sentence_length(sent));
 						num_linkages = sentence_parse(sent, opts);
+						if (num_linkages < 0)
+						{
+							sentence_delete(sent);
+							sent = NULL;
+							continue;
+						}
 					}
 				}
 			}
